// Copyright © 2020 Amazon Web Services
//
// Licensed under the Apache License, Version 2.0 (the "License");
// you may not use this file except in compliance with the License.
// You may obtain a copy of the License at
//
//      http://www.apache.org/licenses/LICENSE-2.0
//
// Unless required by applicable law or agreed to in writing, software
// distributed under the License is distributed on an "AS IS" BASIS,
// WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
// See the License for the specific language governing permissions and
// limitations under the License.

package org.opengroup.osdu.register.provider.aws.action;

import java.sql.Timestamp;

import org.opengroup.osdu.register.action.model.Action;
import org.opengroup.osdu.register.action.model.Filter;

import com.amazonaws.services.dynamodbv2.datamodeling.DynamoDBAttribute;
import com.amazonaws.services.dynamodbv2.datamodeling.DynamoDBHashKey;
import com.amazonaws.services.dynamodbv2.datamodeling.DynamoDBTable;
import com.amazonaws.services.dynamodbv2.datamodeling.DynamoDBTypeConverted;
import com.amazonaws.services.dynamodbv2.datamodeling.DynamoDBTypeConverter;
import com.fasterxml.jackson.core.type.TypeReference;
import com.fasterxml.jackson.databind.ObjectMapper;

import lombok.AllArgsConstructor;
import lombok.Builder;
import lombok.Data;
import lombok.NoArgsConstructor;
import lombok.SneakyThrows;


@Data
@AllArgsConstructor
@NoArgsConstructor
@Builder
@DynamoDBTable(tableName = "Register.Action")
public class ActionDoc {

    @DynamoDBHashKey(attributeName = "id")
    private String id;

    @DynamoDBAttribute(attributeName = "name")
    private String name;

    @DynamoDBAttribute(attributeName = "description")
    private String description;

    @DynamoDBAttribute(attributeName = "img")
    private String img;

    @DynamoDBAttribute(attributeName = "url")
    private String url;

    @DynamoDBAttribute(attributeName = "contactEmail")
    private String contactEmail;

    @DynamoDBAttribute(attributeName = "createdOnEpoch")
    private String createdOnEpoch;

    @DynamoDBAttribute(attributeName = "dataPartitionId")
    private String dataPartitionId;


    @DynamoDBTypeConverted(converter = ActionDoc.ActionConverter.class)
    @DynamoDBAttribute(attributeName = "filter")
    private Filter filter;



    public static class ActionConverter implements DynamoDBTypeConverter<String, Filter> {

        @SneakyThrows
        @Override
        public String convert(Filter object) {
            ObjectMapper mapper = new ObjectMapper();
            return mapper.writeValueAsString(object);
        }

        @SneakyThrows
        @Override
        public Filter unconvert(String object) {
            ObjectMapper mapper = new ObjectMapper();
            return mapper.readValue(object, new TypeReference<Filter>() {
            });
        }
    }



    public static ActionDoc mapFrom(Action action, String dataPartitionId) {

       ActionDocBuilder actionDocBuilder = new ActionDoc().builder()
                .id(action.getId())
                .name(action.getName())
                .description(action.getDescription())
                .img(action.getImg())
                .url(action.getUrl())
                .contactEmail(action.getContactEmail())
                .filter(action.getFilter())
                .createdOnEpoch((new Timestamp(System.currentTimeMillis())).toString())
                .dataPartitionId(dataPartitionId);

        return actionDocBuilder.build();
    }

    public static Action mapto(ActionDoc actionDoc) {

        Action action = new Action();
        action.setId(actionDoc.getId());
        action.setName(actionDoc.getName());
        action.setDescription(actionDoc.getDescription());
        action.setImg(actionDoc.getImg());
        action.setUrl(actionDoc.getUrl());
        action.setContactEmail(actionDoc.getContactEmail());
        action.setCreatedOnEpoch(Timestamp.valueOf(actionDoc.getCreatedOnEpoch()));
        action.setFilter(actionDoc.getFilter());
        return action;
    }





}


