// Copyright © 2020 Amazon Web Services
//
// Licensed under the Apache License, Version 2.0 (the "License");
// you may not use this file except in compliance with the License.
// You may obtain a copy of the License at
//
//      http://www.apache.org/licenses/LICENSE-2.0
//
// Unless required by applicable law or agreed to in writing, software
// distributed under the License is distributed on an "AS IS" BASIS,
// WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
// See the License for the specific language governing permissions and
// limitations under the License.
package org.opengroup.osdu.register.provider.aws.subscriber;

import com.amazonaws.auth.AWSCredentialsProvider;
import com.amazonaws.client.builder.AwsClientBuilder;
import com.amazonaws.services.kms.AWSKMS;
import com.amazonaws.services.kms.AWSKMSClientBuilder;
import org.opengroup.osdu.core.aws.iam.IAMConfig;

//This class should be moved to os-core-lib-aws. Keeping it here temporarily till testing is complete
public class KmsConfig {

    private String amazonKmsEndpoint;

    private String amazonKmsRegion;

    private AWSCredentialsProvider amazonAWSCredentials;

    public KmsConfig(String amazonKmsEndpoint, String amazonKmsRegion){
        amazonAWSCredentials = IAMConfig.amazonAWSCredentials();
        this.amazonKmsEndpoint = amazonKmsEndpoint;
        this.amazonKmsRegion = amazonKmsRegion;
    }

    public AWSKMS awsKMS() {
        // Generate the KMS client
        return AWSKMSClientBuilder.standard()
                .withCredentials(amazonAWSCredentials)
                .withEndpointConfiguration(amazonKmsEndpointConfiguration())
                .build();
    }

    public AwsClientBuilder.EndpointConfiguration amazonKmsEndpointConfiguration() {
        // Create an endpoint configuration for KMS with region and service endpoint from application.properties
        return new AwsClientBuilder.EndpointConfiguration(
                amazonKmsEndpoint, amazonKmsRegion
        );
    }
}
