// Copyright © 2020 Amazon Web Services
//
// Licensed under the Apache License, Version 2.0 (the "License");
// you may not use this file except in compliance with the License.
// You may obtain a copy of the License at
//
//      http://www.apache.org/licenses/LICENSE-2.0
//
// Unless required by applicable law or agreed to in writing, software
// distributed under the License is distributed on an "AS IS" BASIS,
// WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
// See the License for the specific language governing permissions and
// limitations under the License.
package org.opengroup.osdu.register.provider.aws.subscriber;



import com.amazonaws.services.kms.model.DecryptRequest;
import com.amazonaws.services.kms.model.DecryptResult;
import com.amazonaws.services.kms.model.EncryptRequest;
import com.amazonaws.services.kms.model.EncryptResult;
import org.opengroup.osdu.core.aws.dynamodb.DynamoDBQueryHelper;
import org.opengroup.osdu.core.aws.ssm.ParameterStorePropertySource;
import org.opengroup.osdu.core.aws.ssm.SSMConfig;
import org.opengroup.osdu.core.common.logging.JaxRsDpsLog;
import org.opengroup.osdu.core.common.model.http.DpsHeaders;
import org.opengroup.osdu.register.provider.aws.config.AwsServiceConfig;
import org.springframework.beans.factory.annotation.Autowired;
import com.amazonaws.services.kms.*;
import org.springframework.stereotype.Component;
import org.springframework.stereotype.Repository;
import org.springframework.util.Base64Utils;


import javax.annotation.PostConstruct;
import javax.inject.Inject;
import java.io.UnsupportedEncodingException;
import java.nio.ByteBuffer;
import java.nio.charset.StandardCharsets;
import java.util.Base64;
import java.util.Collections;

@Component
public class KmsHelper {

    @Autowired
    private DpsHeaders dpsHeaders;

    @Autowired
    private JaxRsDpsLog logger;



    @Inject
    private AwsServiceConfig serviceConfig;

    private AWSKMS kmsClient;

    private String kmsKeyId;



    @PostConstruct
    public void init() {

        kmsClient = serviceConfig.getKmsClient();
        kmsKeyId=serviceConfig.getKmsKeyId();


    }



   public ByteBuffer encrypt(String plainTextString) {


        EncryptRequest encReq = new EncryptRequest();
        encReq.setKeyId(kmsKeyId);
        encReq.setPlaintext(ByteBuffer.wrap(plainTextString.getBytes()));
        encReq.setEncryptionContext(Collections.singletonMap("dataPartitionId", dpsHeaders.getPartitionId()));
        ByteBuffer ciphertext = kmsClient.encrypt(encReq).getCiphertextBlob();
        return ciphertext;



    }
    public String decrypt(ByteBuffer ciphertext) {

        DecryptRequest decReq = new DecryptRequest();

        decReq.setCiphertextBlob(ciphertext);
        decReq.setEncryptionContext(Collections.singletonMap("dataPartitionId", dpsHeaders.getPartitionId()));
        ByteBuffer decrypted = kmsClient.decrypt(decReq).getPlaintext();

        String decryptedStr = new String(decrypted.array());
        return decryptedStr;
    }

}
