// Copyright 2017-2021, Schlumberger
//
// Licensed under the Apache License, Version 2.0 (the "License");
// you may not use this file except in compliance with the License.
// You may obtain a copy of the License at
//
//      http://www.apache.org/licenses/LICENSE-2.0
//
// Unless required by applicable law or agreed to in writing, software
// distributed under the License is distributed on an "AS IS" BASIS,
// WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
// See the License for the specific language governing permissions and
// limitations under the License.

package org.opengroup.osdu.register.provider.azure.subscriber;

import java.lang.reflect.Type;
import java.util.ArrayList;
import java.util.HashMap;
import java.util.List;
import java.util.Map;

import lombok.RequiredArgsConstructor;
import org.opengroup.osdu.core.common.logging.JaxRsDpsLog;
import org.opengroup.osdu.core.common.model.http.DpsHeaders;
import org.opengroup.osdu.core.common.model.notification.Topic;
import org.opengroup.osdu.register.provider.interfaces.subscriber.ITopicsRepository;
import org.opengroup.osdu.register.utils.FileUtil;
import org.springframework.context.annotation.Primary;
import org.springframework.stereotype.Repository;

import com.google.gson.Gson;
import com.google.gson.reflect.TypeToken;

@Repository
@Primary
@RequiredArgsConstructor
public class TopicsRepositoryImpl implements ITopicsRepository {

    private final JaxRsDpsLog log;
    private final DpsHeaders headers;
    private Map<String, List<Topic>> topicsMap = new HashMap<>();

    public List<Topic> listMessages() {
        final String partitionId = headers.getPartitionId();
        if (topicsMap.containsKey(partitionId)) {
            return topicsMap.get(partitionId);
        }

        final List<Topic> template = getTopicTemplate();

        topicsMap.put(partitionId, template);
        return template;
    }

    List<Topic> getTopicTemplate() {
        Type type = new TypeToken<ArrayList<Topic>>() {
        }.getType();
        return new Gson().fromJson(FileUtil.readFile("topics.json", log), type);
    }
}
